/*-----------------------------------------------------------------------------------------------------*/
/* 『思考エンジン』のサンプル	2010/02/06 作成  2019/06/23 改訂 */

#include <stdio.h>
#include <windows.h>

/*-----------------------------------------------------------------------------------------------------*/
/* メモ書き */

/*
『思考エンジン』の簡単なサンプルプログラムです。

Visual Stadio 2019での利用を想定して記述しましたが、基本的には他の環境でも利用できると思います。
Win32 APIを使ったコードですが、MFCや.Net系でも扱い方は似たり寄ったりだと思います。

このソースコードはCで書かれていますがC++でコンパイルしても問題ないように記述されています。
ソースコードのうち以下のような記述部分はCでコンパイルしてもC++でコンパイルしても通るようにするためのものです。
#ifdef __cplusplus
extern "C" {
#endif

コード

#ifdef __cplusplus
}
#endif

ソースコードは流れを見やすくするためにエラー処理をほとんど行っていません。
実際にはきちんと要所でエラー処理を行ってください。
*/

/*-----------------------------------------------------------------------------------------------------*/
/* 宣言 */

/* load関数の引数として与えられる3つの関数の定義 */
typedef int (*FREE_FUNC)( void * , char * , char * , int );
typedef void *(*MALLOC_FUNC)( int , char * , char * , int );
typedef int (*COM_FUNC)( char  ** , int * );
typedef int (*STATUS_FUNC)(char*);

/* gicp_malloc関数とgicp_free関数のマクロ */
/* これにより、毎回モジュール名やソースファイル、ラインを入力しないで使えます。 */
#define gicp_malloc(s) gicp_malloci( (s) , "CHARACTER" , __FILE__ , __LINE__ )
#define gicp_free(p) gicp_freei( (p) , "CHARACTER" , __FILE__ , __LINE__ )

/* core関数の第1引数用 1がLOAD時、2がUNLOAD時 */
#define STATUS_LOAD	1
#define STATUS_UNLOAD 2

/*変数*/
HINSTANCE g_hInstance;		/*インスタンスハンドル*/
FREE_FUNC gicp_freei;		/*FREE_FUNC関数へのポインタ*/
MALLOC_FUNC gicp_malloci;	/*MALLOC_FUNC関数へのポインタ*/
COM_FUNC gicp_com;			/*COM_FUNC関数へのポインタ*/
STATUS_FUNC gicp_status;	/*STATUS_FUNC関数へのポインタ*/

/*プロトタイプ宣言*/
int gicp_communication( char ** , int * );
int gicp_switch( char * );
int gicp_sendcom( char ** , int * );
char *gicp_setstring( char * , char * , char * , char * );
int gicp_noticeexecution_start( char ** , int * );
int gicp_noticeexecution_exit( char ** , int * );
int gicp_noticeexecution_click( char ** , int * );
int gicp_noticeexecution_talkfinish( char ** , int * );
int gicp_notsupport( char ** , int * );
int charcode_sjis2utf( char * , char * , int );
int charcode_utf2sjis( char * , char * , int );

/* 関数のEXPORT */
#ifdef __cplusplus
extern "C" {
#endif

	__declspec(dllexport) int load( int , char * , void * , MALLOC_FUNC , FREE_FUNC , COM_FUNC , STATUS_FUNC);
	__declspec(dllexport) int communication( char ** , int * );
	__declspec(dllexport) int configuration( HWND , unsigned short );

#ifdef __cplusplus
}
#endif

/*-----------------------------------------------------------------------------------------------------*/
/* プログラム */

//-------------------------------------------------------------------------------------------------------
//
//Dllのメイン関数

BOOL APIENTRY DllMain( HANDLE hModule , DWORD  ul_reason_for_call , LPVOID lpReserved  )
{
	switch(ul_reason_for_call)
	{
	case    DLL_PROCESS_ATTACH:
		g_hInstance = (HINSTANCE)hModule;				//インスタンスハンドルをグローバルにコピー
		break; 
	case    DLL_PROCESS_DETACH:
		break;
	default:
		break;
	}
	return TRUE;
}

/*-------------------------------------------------------------------------------------------------------*/
/* エクスポートする関数 */

int load( int status , char *in_path , void *reserved , MALLOC_FUNC in_mallocfunc , FREE_FUNC in_freefunc , COM_FUNC in_comfunc , STATUS_FUNC in_statusfunc )
{
	switch(status)
	{
	case STATUS_LOAD:
		{
			/* 各変数をグローバル変数へコピー */
			gicp_malloci = in_mallocfunc;
			gicp_freei = in_freefunc;
			gicp_com = in_comfunc;
			gicp_status = in_statusfunc;
			return TRUE;
		}
		break;
	case STATUS_UNLOAD:
		{
			return TRUE;
		}
		break;
	default:
		return FALSE;
		break;
	}
}

int configuration( HWND hWnd , unsigned short us_language )
{
	/* ここでは作成しませんが、利用する場合はDialogBox(又はDialogBoxParam)でダイアログボックスを作ってください。*/
	return FALSE;
}

int communication( char **lpp_buffer , int *lp_length )
{
	char *lp_sjis;
	int length;

	/* UTF-8文字列をShift_JISに変換する場合に必要な文字数("\0"含む) */
	length = charcode_utf2sjis( *lpp_buffer , NULL , 0 );
	/* メモリを確保 */
	lp_sjis = (char*)malloc( length*sizeof(char) );
	/* UTF-8文字列をShift_JISに変換して格納 */
	charcode_utf2sjis( *lpp_buffer , lp_sjis , length );
	/* 文字列の長さを取得 */

	/* GICPを処理する関数に送る */
	if(!gicp_communication( &lp_sjis , &length ))
	{
		/* GICP通信失敗 */
		free( lp_sjis );
		return FALSE;
	}

	/* GICPのメモリを解放 */
	gicp_free( *lpp_buffer );

	/* Shift_JIS文字列をUTF-8に変換する場合に必要な文字数("\0"含む) */
	length = charcode_sjis2utf( lp_sjis , NULL , 0 );
	/* GICPのメモリを確保 */
	*lpp_buffer = (char*)gicp_malloc( length*sizeof(char) );
	/* Shift_JIS文字列をUTF-8に変換して格納 */
	charcode_sjis2utf( lp_sjis , *lpp_buffer , length );

	/* メモリ解放 */
	free( lp_sjis );

	/* バッファサイズを代入 */
	*lp_length = length;
	return TRUE;
}

/*-------------------------------------------------------------------------------------------------------*/
/* GICPを処理する関数 */

/* GICP命令の大元の関数、命令ごとに割り振るって各関数に送る。 */
int gicp_communication( char **in_buffer , int *in_length )
{
	switch(gicp_switch( *in_buffer ))
	{
	case 1:
		return gicp_noticeexecution_start( in_buffer , in_length );
		break;
	case 2:
		return gicp_noticeexecution_exit( in_buffer , in_length );
		break;
	case 3:
		return gicp_noticeexecution_click( in_buffer , in_length );
		break;
	case 4:
		return gicp_noticeexecution_talkfinish( in_buffer , in_length );
		break;
	default:
		return gicp_notsupport( in_buffer , in_length );
		break;
	}
}

/* 分岐する関数 */
/* これは非常に簡易な関数なので、本来はもう少しきちんとしたものを作る必要があると思います。*/
int gicp_switch( char *in_buffer )
{
	if (strstr(in_buffer, "<class>NOTICE</class><order>EXECUTION</order>"))
	{
		/* NOTICE-EXECUTION 命令 */

		if (strstr(in_buffer, "<information>START</information><parameter></parameter>"))
		{
			/* START */
			return 1;
		}
		else if (strstr(in_buffer, "<information>EXIT</information><parameter></parameter>"))
		{
			/* EXIT */
			return 2;
		}
		else if (strstr(in_buffer, "<information>CLICK</information>"))
		{
			/* CLICK */
			/* パラメータはクリックした場所によって変わるのチェック対象から外しています。*/
			return 3;
		}
		else if (strstr(in_buffer, "<information>TALK</information><parameter>FINISH</parameter>"))
		{
			/* CANVAS-FINISH 命令 */
			return 4;
		}
		else
		{
			return 0;
		}



		return 1;
	}
}

/* GICP文字列を送信する */
int gicp_sendcom( char **in_buffer , int *in_length )
{
	int length;
	char *lp_utf8;

	/* Shift_JIS文字列をUTF-8に変換する場合に必要な文字数("\0"含む) */
	length = charcode_sjis2utf( *in_buffer , NULL , 0 );
	/* GICPのメモリ確保 */
	lp_utf8 = (char*)gicp_malloc( length*sizeof(char) );
	/* Shift_JIS文字列をUTF-8に変換して格納 */
	charcode_sjis2utf( *in_buffer , lp_utf8 , length );

	if(!gicp_com( &lp_utf8 , &length ))
	{
		/* GICP通信失敗 */
		gicp_free( lp_utf8 );	/* GICPのメモリを解放 */
		return FALSE;
	}

	/* メモリを解放 */
	free( *in_buffer );
	/* UTF-8文字列をShift_JISに変換する場合に必要な文字数("\0"含む) */
	length = charcode_utf2sjis( lp_utf8 , NULL , 0 );
	/* メモリ確保 */
	*in_buffer = (char*)malloc( length*sizeof(char) );
	/* UTF-8文字列をShift_JISに変換して格納 */
	charcode_utf2sjis( lp_utf8 , *in_buffer , length );

	/* GICPのメモリを解放 */
	gicp_free( lp_utf8 );

	/* バッファサイズを代入 */
	*in_length = length;
	return TRUE;
}

/* 各データを元にGICP文字列を作る関数 */
char *gicp_setstring( char *in_master , char *in_follower , char *in_witchcraft , char *in_incantation )
{
	char *buffer;
	int length;

	/* バッファのサイズを計算する */
	length = strlen( "<\?xml version=\"1.0\" encoding=\"UTF-8\"\?>\r\n<gicp version=\"1.0\">\r\n<master></master>\r\n<follower></follower>\r\n<witchcraft></witchcraft>\r\n<incantation></incantation>\r\n</gicp>" );
	length = length + strlen(in_master);
	length = length + strlen(in_follower);
	length = length + strlen(in_witchcraft);
	length = length + strlen(in_incantation);	

	/* バッファを確保する */
	buffer = (char*)malloc( (length+1)*sizeof(char) );

	/* バッファにも文字列をコピーする */
	sprintf( buffer , "<\?xml version=\"1.0\" encoding=\"UTF-8\"\?>\r\n<gicp version=\"1.0\">\r\n<master>%s</master>\r\n<follower>%s</follower>\r\n<witchcraft>%s</witchcraft>\r\n<incantation>%s</incantation>\r\n</gicp>" , in_master , in_follower , in_witchcraft , in_incantation );
	return buffer;
}

/* NOTICE-START命令を処理する関数 */
int gicp_noticeexecution_start( char **in_buffer , int *in_length )
{
	char master[256];
	char follower[256];
	char witchcraft[256];
	char incantation[512];
	char *script;
	char *buffer;
	int length;


	/* -- ここからセリフを表示するための処理 -- */

	/* スクリプト */
	/* \\f[smile]はキャラクターに『smile.png』というファイルがあった場合の例です。*/
	script = "\\m[smile]起動しました！";

	/* 各要素を作る */
	/* このサンプルでの識別用にuserdataに<param>START</param>を加えています。 */
	sprintf( master , "<place>CHARACTER</place><detail></detail>" );
	sprintf( follower , "<place>MASCOT</place><detail></detail>" );
	sprintf( witchcraft , "<class>TALK</class><order>EXECUTION</order>" );
	sprintf( incantation , "<priority>DAUGHTER</priority><script>%s</script><materials></materials><userdata><event><userdata><param>START</param></userdata>" , script );

	/* 各データを元にGICP文字列を作る */
	buffer = gicp_setstring( master , follower , witchcraft , incantation );
	length = strlen(buffer)+1;

	/* GICPを送信！*/
	gicp_sendcom( &buffer , &length );

	/* メモリを解放 */
	free( buffer );

	/* -- セリフを表示するための処理ここまで-- */


	/* あとはGICP RETURNを返して終了 */
	/* 各要素を作る */
	sprintf( master ,  "<place>SOMEONE</place><detail></detail>" );
	sprintf( follower ,  "<place>SOMEONE</place><detail></detail>" );
	sprintf( witchcraft , "<class>RETURN</class><order>RETURN</order>" );
	sprintf( incantation , "" );

	/* 各データを元にGICP文字列を作る */
	buffer = gicp_setstring( master , follower , witchcraft , incantation );
	length = strlen(buffer)+1;

	/* 元のメモリ解放してから代入 */
	free( *in_buffer );
	*in_buffer = buffer;
	*in_length = length;
	return TRUE;
}

/* NOTICE-END命令を処理する関数 */
int gicp_noticeexecution_exit( char **in_buffer , int *in_length )
{
	char master[256];
	char follower[256];
	char witchcraft[256];
	char incantation[256];
	char *buffer;
	int length;


	/* -- ここからマスコット本体を終了させる命令を出す処理 -- */

	/* 各要素を作る */
	sprintf( master , "<place>CHARACTER</place><detail></detail>" );
	sprintf( follower , "<place>MASCOT</place><detail></detail>" );
	sprintf( witchcraft , "<class>SYSTEM</class><order>EXIT</order>" );
	sprintf( incantation , "" );

	/* 各データを元にGICP文字列を作る */
	buffer = gicp_setstring( master , follower , witchcraft , incantation );
	length = strlen(buffer)+1;

	/* GICPを送信！*/
	gicp_sendcom( &buffer , &length );

	/* メモリを解放 */
	free( buffer );

	/* -- マスコット本体を終了させる命令を出す処理ここまで -- */


	/* あとはGICP RETURNを返して終了 */
	/* 各要素を作る */
	sprintf( master , "<place>SOMEONE</place><detail></detail>" );
	sprintf( follower , "<place>SOMEONE</place><detail></detail>" );
	sprintf( witchcraft , "<class>RETURN</class><order>RETURN</order>" );
	sprintf( incantation , "" );

	/* 各データを元にGICP文字列を作る */
	buffer = gicp_setstring( master , follower , witchcraft , incantation );
	length = strlen(buffer)+1;

	/* 元のメモリ解放してから代入 */
	free( *in_buffer );
	*in_buffer = buffer;
	*in_length = length;
	return TRUE;
}

/* NOTICE-CLICK命令を処理する関数 */
int gicp_noticeexecution_click( char **in_buffer , int *in_length )
{
	char master[256];
	char follower[256];
	char witchcraft[256];
	char incantation[512];
	char *script;
	char *buffer;
	int length;


	/* -- ここからセリフを表示するための処理 -- */

	/* スクリプト */
	/* \\f[smile]はキャラクターに『smile.png』というファイルがあった場合の例です。*/
	script = "\\m[smile]クリックされました！";

	/* 各要素を作る */
	/* このサンプルでの識別用にuserdataに<param>LEFTCLICK</param>を加えています。 */
	sprintf( master , "<place>CHARACTER</place><detail></detail>" );
	sprintf( follower , "<place>MASCOT</place><detail></detail>" );
	sprintf( witchcraft , "<class>TALK</class><order>EXECUTION</order>" );
	sprintf( incantation , "<priority>DAUGHTER</priority><script>%s</script><materials></materials><userdata><param>CLICK</param></userdata>" , script );

	/* 各データを元にGICP文字列を作る */
	buffer = gicp_setstring( master , follower , witchcraft , incantation );
	length = strlen(buffer)+1;

	/* GICPを送信！*/
	gicp_sendcom( &buffer , &length );

	/* メモリを解放 */
	free( buffer );

	/* -- セリフを表示するための処理ここまで-- */


	/* あとはGICP RETURNを返して終了 */
	/* 各要素を作る */
	sprintf( master ,  "<place>SOMEONE</place><detail></detail>" );
	sprintf( follower ,  "<place>SOMEONE</place><detail></detail>" );
	sprintf( witchcraft , "<class>RETURN</class><order>RETURN</order>" );
	sprintf( incantation , "" );

	/* 各データを元にGICP文字列を作る */
	buffer = gicp_setstring( master , follower , witchcraft , incantation );
	length = strlen(buffer)+1;

	/* 元のメモリ解放してから代入 */
	free( *in_buffer );
	*in_buffer = buffer;
	*in_length = length;
	return TRUE;
}

/* TALK-FINISH命令を処理する関数 */
int gicp_noticeexecution_talkfinish( char **in_buffer , int *in_length )
{
	char master[256];
	char follower[256];
	char witchcraft[256];
	char incantation[512];
	char *buffer;
	int length;

	char *p;

	p = strstr( *in_buffer , "<incantation>" );

	if(strstr(p,"<userdata><param>START</param></userdata>"))
	{
		/* スタートの台詞が終了した */
		/* 最後まで表示されたか、途中でキャンセルされたかなども調べることができる */
	}
	else if(strstr(p,"<userdata><param>CLICK</param></userdata>"))
	{
		/* クリックの台詞が終了した */
		/* 最後まで表示されたか、途中でキャンセルされたかなども調べることができる */
	}

	/* あとはGICP RETURNを返して終了 */
	/* 各要素を作る */
	sprintf( master ,  "<place>SOMEONE</place><detail></detail>" );
	sprintf( follower ,  "<place>SOMEONE</place><detail></detail>" );
	sprintf( witchcraft , "<class>RETURN</class><order>RETURN</order>" );
	sprintf( incantation , "<result>OK</result><id>smile</id>" );
	buffer = gicp_setstring( master , follower , witchcraft , incantation );
	length = strlen(buffer)+1;

	/* 元のメモリ解放してから代入 */
	free( *in_buffer );
	*in_buffer = buffer;
	*in_length = length;
	return TRUE;
}

/* 未対応の命令が来た場合の対処 */
int gicp_notsupport( char **in_buffer , int *in_length )
{
	char master[256];
	char follower[256];
	char witchcraft[256];
	char incantation[512];
	char *buffer;
	int length;

	/* 各要素を作る */
	sprintf( master ,  "<place>SOMEONE</place><detail></detail>" );
	sprintf( follower ,  "<place>SOMEONE</place><detail></detail>" );
	sprintf( witchcraft , "<class>RETURN</class><order>RETURN</order>" );
	sprintf( incantation ,  "<result>NOTSUPPORTED</result>" );

	/* 各データを元にGICP文字列を作る */
	buffer = gicp_setstring( master , follower , witchcraft , incantation );
	length = strlen(buffer)+1;

	/* 元のメモリ解放してから代入 */
	free( *in_buffer );
	*in_buffer = buffer;
	*in_length = length;
	return TRUE;
}

/*-------------------------------------------------------------------------------------------------------*/
/* 文字コードを処理する関数 */

/* UTF-8をShift_JISに直接変換する関数はないが、Unicode(UTF-16)を介することで比較的簡単に実現できる */
/* buffer_sizeが0の時は必要なメモリ量を戻り値として返し、0以外の場合はlp_utfに変換して格納する */

/* Shift_JISからUTF-8に変換する */
int charcode_sjis2utf( char *lp_sjis , char *lp_utf8 , int buffer_size )
{
	wchar_t *lp_unicode;
	int length;

	/* UTF-8文字列をUnicodeに変換する場合に必要な文字数("\0"含む) */
	length = MultiByteToWideChar( CP_ACP , 0 , lp_sjis , -1 , NULL , 0 );
	/* メモリを確保 */
	lp_unicode = (wchar_t*)malloc( length*sizeof(wchar_t) );
	/* UTF-8文字列をUnicodeに変換して格納 */
	MultiByteToWideChar( CP_ACP , 0 , lp_sjis , -1 , lp_unicode , length );

	if(buffer_size==0)
	{
		/* Unicode文字列をUTF-8に変換する場合に必要な文字数("\0"含む) */
		length = WideCharToMultiByte( CP_UTF8 , 0 , lp_unicode , -1 , NULL , 0 , NULL , NULL );
		free( lp_unicode );
		return length;
	}
	else
	{
		/* Unicode文字列をUTF-8に変換して格納 */
		length = WideCharToMultiByte( CP_UTF8 , 0 , lp_unicode , -1 , lp_utf8 , buffer_size , NULL , NULL );
		free( lp_unicode );
		return length;
	}
}

/* UTF-8からShift_JISに変換する */
int charcode_utf2sjis( char *lp_utf8 , char *lp_sjis , int buffer_size )
{
	wchar_t *lp_unicode;
	int length;

	/* UTF-8文字列をUnicodeに変換する場合に必要な文字数("\0"含む) */
	length = MultiByteToWideChar( CP_UTF8 , 0 , lp_utf8 , -1 , NULL , 0 );
	/* メモリを確保 */
	lp_unicode = (wchar_t*)malloc( length*sizeof(wchar_t) );
	/* UTF-8文字列をUnicodeに変換して格納 */
	MultiByteToWideChar( CP_UTF8 , 0 , lp_utf8 , -1 , lp_unicode , length );

	if(buffer_size==0)
	{
		/* Unicode文字列をUTF-8に変換する場合に必要な文字数("\0"含む) */
		length = WideCharToMultiByte( CP_ACP , 0 , lp_unicode , -1 , NULL , 0 , NULL , NULL );
		free( lp_unicode );
		return length;
	}
	else
	{
		/* Unicode文字列をUTF-8に変換して格納 */
		length = WideCharToMultiByte( CP_ACP , 0 , lp_unicode , -1 , lp_sjis , buffer_size , NULL , NULL );
		free( lp_unicode );
		return length;
	}
}