/*-----------------------------------------------------------------------------------------------------*/
/* TCP/IPで連携するプログラムのサンプル	2019/06/23作成 */

#include <stdio.h>
#include <windows.h>
#include <process.h>	/* スレッド処理で使う */

/* winsockで利用するライブラリ */
#pragma comment(lib, "wsock32.lib")

/*-----------------------------------------------------------------------------------------------------*/
/* メモ書き */

/*
オプションプログラムの簡単なサンプルコードです。

Visual Stadio 2019 での利用を想定して記述しましたが、基本的には他の環境でも利用できると思います。
Win32 APIを使ったコードですが、MFCや.Net系でも扱い方は似たり寄ったりだと思います。

このソースコードはCで書かれていますがC++でコンパイルしても問題ないように記述されています。
ソースコードのうち以下のような記述部分はCでコンパイルしてもC++でコンパイルしても通るようにするためのものです。
#ifdef __cplusplus
extern "C" {
#endif

コード

#ifdef __cplusplus
}
#endif

ソースコードは流れを見やすくするためにエラー処理をほとんど行っていません。
実際にはきちんと要所でエラー処理を行ってください。
ネットワーク部分はよくエラーが起こりえるのでちょっとだけエラー処理もしています。

このプログラムはスレッドを使っているので、ランタイムライブラリはマルチスレッド系を選んで下さい。
*/

/*-----------------------------------------------------------------------------------------------------*/
/* 宣言 */

/* メニューのID登録 */
#define ID_SEND 1
#define ID_EXIT 2

/* バッファサイズ(アバウトな数字) */
#define MAX_STRING 1024

/* TCP/IPで送ることができる最大のバイト数 */
#define MAX_STRING_TCP 8192

/* Winsockで使う */
#define SD_BOTH 0x02

/* 子ウィンドウのID */
#define ID_STATIC 1

/* プロトタイプ宣言 */
LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
int server( int );
unsigned int __stdcall server_thread( void * );
char *winsock_recv( SOCKET );
int winsock_send( SOCKET , char * );
int gicp_recv( SOCKET );
int gicp_send( char * , unsigned short , char * );

/* サーバーのフラグ */
int flag = FALSE;

/* ウィンドウハンドル(グローバル変数) */
HWND g_hwnd;

/* メイン関数 */
int APIENTRY WinMain( HINSTANCE hInstance , HINSTANCE hPrevInstance , LPTSTR lpCmdLine , int nCmdShow )
{
	MSG msg;
	WNDCLASSEX wcex;
	HWND hWnd;
	WSADATA wsadata;

	/* winsock初期化 MAKEWORD(1,1)はバージョン */
	memset(&wsadata,0,sizeof(WSADATA));
	WSAStartup(MAKEWORD(1,1),&wsadata);

	/* ウィンドウクラスの登録 */
	memset(&wcex,0,sizeof(WNDCLASSEX));
	wcex.cbSize			= sizeof(WNDCLASSEX);
	wcex.style			= CS_HREDRAW|CS_VREDRAW;
	wcex.lpfnWndProc	= WndProc;
	wcex.hInstance		= hInstance;
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszClassName	= "WindowClass";
	RegisterClassEx(&wcex);

	/* ウィンドウの作成 */
	hWnd = CreateWindow( "WindowClass" , "Sample" , WS_OVERLAPPEDWINDOW , CW_USEDEFAULT , 0 , 240 , 100 , NULL , NULL , hInstance , NULL );
	ShowWindow(hWnd, SW_SHOW );
	UpdateWindow(hWnd);

	/* ウィンドウメッセージループ */
	while(GetMessage(&msg, NULL, 0, 0)>0)
	{
		DispatchMessage(&msg);
	}

	/* winsock 終了 */
	WSACleanup();
	return (int)msg.wParam;
}

/* ウィンドウのプロシージャー */
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_CREATE:
		{
			HMENU hMenu;
			HMENU hSubMenu;

			/* ウィンドウハンドルをグローバルへコピー */
			g_hwnd = hWnd;

			/* メニューの作成 */
			hMenu = CreateMenu();
			hSubMenu = CreateMenu();

			AppendMenu( hMenu , MF_POPUP, (UINT_PTR)hSubMenu , "メニュー" );
			AppendMenu( hSubMenu , MF_STRING , ID_SEND , "GICP命令を送る" );
			AppendMenu( hSubMenu , MF_STRING , ID_EXIT , "終了" );
			SetMenu( hWnd , hMenu );

			/* サーバー起動 */
			server( 1 );
			return TRUE;
		}
		break;
	case WM_SIZE:
		{
			/* 親ウィンドウの大きさに合わせて子ウィンドウをリサイズ */
			RECT rect;
			GetClientRect(hWnd,&rect);
			MoveWindow(GetDlgItem(hWnd,ID_STATIC),0,0,rect.right-rect.left,rect.bottom-rect.top,FALSE);
			return TRUE;
		}
		break;
	case WM_COMMAND:
		{
			/* コマンド(メニュー)処理 */
			unsigned short wmId, wmEvent;
			wmId    = LOWORD(wParam);
			wmEvent = HIWORD(wParam);

			switch (wmId)
			{
			case ID_SEND:
				{
					/* GICP命令を送るが選択された場合の処理 */
					char *master;
					char *follower;
					char *witchcraft;
					char *incantation;
					char gicp[MAX_STRING];
					char ip[4];

					/* IPアドレスとポートは適切なものを指定してください */
					/* この例ではマルチバイト to UTF8 の変換を行っていません */
					/* ASCII文字以外を送信する場合は変換してください */
					master = "<master><place>NETWORK</place><detail><address>127.0.0.1:55556</address></detail></master>";
					follower = "<follower><place>MASCOT</place><detail></detail></follower>";
					witchcraft = "<witchcraft><class>NOTICE</class><order>REQUEST</order></witchcraft>";
					incantation = "<incantation><event><information>MESSAGE</information><parameter></parameter></event><data><sender>CLIENT</sender><string>The sample client send a message.</string><priority>DAUGHTER</priority></data><userdata></userdata></incantation>";
					sprintf( gicp , "<\?xml version=\"1.0\" encoding=\"UTF-8\"\?>\r\n<gicp version=\"1.0\">\r\n%s\r\n%s\r\n%s\r\n%s\r\n</gicp>" , master , follower , witchcraft , incantation );

					/* 送信！ */
					/* IPアドレスとポートは適切なものを指定してください */
					ip[0] = 127;
					ip[1] = 0;
					ip[2] = 0;
					ip[3] = 1;
					if(gicp_send(ip,55555,gicp))
					{
						MessageBox(hWnd,"成功","情報",MB_OK);
					}
					else
					{
						MessageBox(hWnd,"失敗","情報",MB_OK);
					}
					return TRUE;
				}
				break;
			case ID_EXIT:
				{
					/* 終了が選択された場合の処理 */
					DestroyWindow( hWnd );
					return TRUE;
				}
				break;
			default:
				return DefWindowProc(hWnd, message, wParam, lParam);
				break;
			}
		}
		break;
	case WM_PAINT:
		{
			/* 描画 */
			/* このプログラムの本質とは関係なし。 */
			PAINTSTRUCT ps;
			HDC hDC;
			hDC = BeginPaint(hWnd, &ps);
			EndPaint(hWnd, &ps);
			return TRUE;
		}
		break;
	case WM_DESTROY:
		{
			/* サーバー終了 */
			server( 2 );

			/* メッセージループの終了 */
			PostQuitMessage(0);
			return 0;
		}
		break;
	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
		break;
	}
}

int server( int param )
{
	static SOCKET sock = 0;
	static HANDLE handle = NULL;

	switch( param )
	{
	case 1:
		{
			SOCKADDR_IN address;
			unsigned int value;

			memset(&address,0,sizeof(SOCKADDR_IN));
			sock = socket(AF_INET,SOCK_STREAM,0);
			if(sock== INVALID_SOCKET)
			{
				return FALSE;
			}

			address.sin_family = AF_INET;
			address.sin_port = htons(55556);	/* 55556はポート番号 */
			if(bind(sock,(SOCKADDR*)(&address),sizeof(SOCKADDR))!=0)
			{
				return FALSE;
			}

			if(listen(sock,1)!=0)
			{
				return FALSE;
			}						

			flag = TRUE;	/* グローバル変数 */
			handle = (HANDLE)_beginthreadex( NULL , 0 , server_thread , (void*)(&sock) , 0 , &value );
			return TRUE;
		}
		break;
	case 2:
		{
			flag = FALSE;	/* グローバル変数 */
			shutdown(sock,SD_BOTH);
			closesocket(sock);

			if(WaitForSingleObject(handle,10)==WAIT_TIMEOUT)
			{
				/* タイムアイト */
			}
			CloseHandle(handle);
			return TRUE;
		}
		break;
	default:
		return FALSE;
		break;
	}
}

unsigned int __stdcall server_thread( void *lpvoid )
{
	SOCKET in_sock;

	in_sock = *((SOCKET*)lpvoid);
	while(flag)
	{
		gicp_recv(in_sock);
	}
	_endthreadex(0);
	return TRUE;
}

int gicp_recv( SOCKET in_sock )
{
	SOCKADDR_IN client;
	int address_size;
	SOCKET sock;

	char *buffer;

	char *master;
	char *follower;
	char *witchcraft;
	char *incantation;
	char gicp[MAX_STRING];


	address_size = sizeof(SOCKADDR_IN);
	memset(&client,0, sizeof(SOCKADDR_IN));

	sock = accept(in_sock,(SOCKADDR*)(&client),&address_size);

	/* 受信！ */
	buffer = winsock_recv(sock);
	if(!buffer)
	{
		/* エラー */
		shutdown(sock , SD_BOTH);
		closesocket(sock);
		return FALSE;
	}

	/* 受信したデータをウィンドウ(中に作った子ウィンドウ)に表示 */
	SetWindowText(GetDlgItem(g_hwnd,ID_STATIC),buffer);

	/* bufferはmallocで確保されているので必ず解放してください */
	free( buffer );

	master = "<master><place>SOMEONE</place><detail></detail></master>";
	follower = "<follower><place>SOMEONE</place><detail></detail></follower>";
	witchcraft = "<witchcraft><class>RETURN</class><order>RETURN</order></witchcraft>";
	incantation = "<incantation></incantation>";
	sprintf( gicp , "<\?xml version=\"1.0\" encoding=\"UTF-8\"\?>\r\n<gicp version=\"1.0\">\r\n%s\r\n%s\r\n%s\r\n%s\r\n</gicp>" , master , follower , witchcraft , incantation );

	/* 返信！ */
	if(!winsock_send(sock,gicp))
	{
		/* 返事がないただの屍のようだ */
		shutdown(sock,SD_BOTH);
		closesocket(sock);
		return FALSE;
	}

	shutdown(sock,SD_BOTH);
	closesocket(sock);
	return TRUE;
}

int gicp_send( char *ip , unsigned short port , char *lp_buffer )
{
	SOCKET sock;
	SOCKADDR_IN sockaddr;
	HOSTENT *lp_hostent;
	char *lp_result;

	memset( &sockaddr , 0 , sizeof(SOCKADDR_IN) );

	lp_hostent = gethostbyaddr(ip,4,AF_INET);
	if(!lp_hostent)
	{
		/* エラー */
		return FALSE;
	}

	/* ソケットの作成 */
	sock = socket(AF_INET,SOCK_STREAM,0);
	sockaddr.sin_family = AF_INET;
	sockaddr.sin_port = htons( port );
	sockaddr.sin_addr = *((struct in_addr*)(*(lp_hostent->h_addr_list)));

	/* 接続 */
	if(connect(sock,(LPSOCKADDR)&sockaddr,sizeof(SOCKADDR_IN))!=0)
	{
		return FALSE;
	}

	if(!winsock_send(sock,lp_buffer))
	{
		/* エラー */
		shutdown(sock , SD_BOTH);
		closesocket(sock);
		return FALSE;
	}

	lp_result = winsock_recv(sock);
	if(!lp_result)
	{
		/* 返事がないただの屍のようだ */
		shutdown(sock,SD_BOTH);
		closesocket(sock);
		return FALSE;
	}

	/* 結果を使うなら使う */
	/* mallocで確保されているので解放を忘れない */
	free( lp_result );

	shutdown(sock,SD_BOTH);
	closesocket(sock);
	return TRUE;
}

int winsock_send( SOCKET sock , char *lp_buffer )
{
	/* 実際にはsendで送ったものがちゃんと送られるとは限らないので */
	/* sendの戻り値を見て処理しなければならない */

	char buffer[MAX_STRING_TCP];
	int length;
	int stack;

	/* 文字列の長さを取得 +1は最後の\0の分 */
	length = strlen(lp_buffer)+1;
	stack = length;

	while(0<stack)
	{
		/* 残りのサイズは送信可能か */
		if(stack < MAX_STRING_TCP )
		{
			/* 送信可能だ */
			memset( buffer , 0 , sizeof(char)*MAX_STRING_TCP );
			memcpy( buffer , lp_buffer+(length-stack) , stack );

			if(send( sock , buffer , stack , 0 )<=0)
			{
				return FALSE;	/*エラー */
			}

			/* 完了 */
			return TRUE;
		}
		else
		{
			/* まとめて送信できないので分割する */
			memset( buffer , 0 , sizeof(char)*MAX_STRING_TCP );
			memcpy( buffer , lp_buffer+(length-stack) , MAX_STRING_TCP );

			if(send( sock , buffer , MAX_STRING_TCP , 0 )<=0)
			{
				return FALSE;	/*エラー */
			}

			/* 残りの処理 */
			stack = stack - MAX_STRING_TCP;
		}
	}

	/* 本来ココには来ないかと */
	return TRUE;
}

char *winsock_recv( SOCKET sock )
{
	char buffer[MAX_STRING_TCP];
	char *lp_buffer;
	int size;
	int length;

	length = 0;
	lp_buffer = NULL;

	while(TRUE)
	{
		/* まず仮に入れるバッファを初期化 */
		memset(buffer,0,sizeof(char)*MAX_STRING_TCP);

		/* 受信 */
		size = recv( sock , buffer , MAX_STRING_TCP , 0 );
		if(size<=0)
		{
			/* もう受信するデータはない=完了 */
			return lp_buffer;
		}

		/* 新たに受信したデータを既に受信したデータに加える */
		/* reallocはlp_buffer=NULLならばmallocとして動作する */
		lp_buffer = (char*)realloc(lp_buffer,length+size);
		memcpy(lp_buffer+length,buffer,size);
		length = length+size;

		/* 最後が\0ならばGICP文字列の最後 */
		/* これはGICPの仕様なので他のプログラムの場合はこの限りではない */
		if(lp_buffer[length-1]=='\0')	
		{
			/* 完了 */
			return lp_buffer;
		}
	}
}