/*-----------------------------------------------------------------------------------------------------*/
/* オプションプログラムのサンプル	2010/02/06 作成  2019/06/23 改訂 */

#include <stdio.h>
#include <windows.h>
#include <process.h>

/*-----------------------------------------------------------------------------------------------------*/
/* メモ書き */

/*
オプションプログラムの簡単なサンプルコードです。

Visual Stadio 2019 での利用を想定して記述しましたが、基本的には他の環境でも利用できると思います。
Win32 APIを使ったコードですが、MFCや.Net系でも扱い方は似たり寄ったりだと思います。

このソースコードはCで書かれていますがC++でコンパイルしても問題ないように記述されています。
ソースコードのうち以下のような記述部分はCでコンパイルしてもC++でコンパイルしても通るようにするためのものです。
#ifdef __cplusplus
extern "C" {
#endif

コード

#ifdef __cplusplus
}
#endif

ソースコードは流れを見やすくするためにエラー処理をほとんど行っていません。
実際にはきちんと要所でエラー処理を行ってください。

このプログラムはスレッドを使っているので、ランタイムライブラリはマルチスレッド系を選んで下さい。
*/

/*-----------------------------------------------------------------------------------------------------*/
/* 宣言 */

/* load関数の引数として与えられる3つの関数の定義 */
typedef int (*FREE_FUNC)( void * , char * , char * , int );
typedef void *(*MALLOC_FUNC)( int , char * , char * , int );
typedef int (*COM_FUNC)( char  ** , int * );
typedef int (*STATUS_FUNC)( char * );

/* gicp_malloc関数とgicp_free関数のマクロ */
/* これにより、毎回モジュール名やソースファイル、ラインを入力しないで使えます。 */
#define gicp_malloc(s) gicp_malloci( (s) , "OPTION" , __FILE__ , __LINE__ )
#define gicp_free(p) gicp_freei( (p) , "OPTION" , __FILE__ , __LINE__ )

/* core関数の第1引数用 1がLOAD時、2がUNLOAD時 */
#define STATUS_LOAD	1
#define STATUS_UNLOAD 2

/*変数*/
HINSTANCE g_hInstance;		/*インスタンスハンドル*/
FREE_FUNC gicp_freei;		/*FREE_FUNC関数へのポインタ*/
MALLOC_FUNC gicp_malloci;	/*MALLOC_FUNC関数へのポインタ*/
COM_FUNC gicp_com;			/*COM_FUNC関数へのポインタ*/
STATUS_FUNC gicp_status;	/*STATUS_FUNC関数へのポインタ*/

/*プロトタイプ宣言*/
int gicp_communication( char ** , int * );
int gicp_switch( char * );
int gicp_sendcom( char ** , int * );
char *gicp_setstring( char * , char * , char * , char * );
int gicp_functiongetfunction( char ** , int * );
int gicp_functionexecution( char ** , int * );
int gicp_noticerequest();
int gicp_notsupport( char ** , int * );
int charcode_sjis2utf( char * , char * , int );
int charcode_utf2sjis( char * , char * , int );
int window_createwindow();
void window_thread( void * );
LRESULT CALLBACK window_procedure(HWND , UINT , WPARAM , LPARAM );

/* 関数のEXPORT */
#ifdef __cplusplus
extern "C" {
#endif

	__declspec(dllexport) int load( int , char * , void * , MALLOC_FUNC , FREE_FUNC , COM_FUNC , STATUS_FUNC );
	__declspec(dllexport) int communication( char ** , int * );
	__declspec(dllexport) int configuration( HWND , unsigned short );

#ifdef __cplusplus
}
#endif

/*-----------------------------------------------------------------------------------------------------*/
/* プログラム */

//-------------------------------------------------------------------------------------------------------
//
//Dllのメイン関数

BOOL APIENTRY DllMain( HANDLE hModule , DWORD  ul_reason_for_call , LPVOID lpReserved  )
{
	switch(ul_reason_for_call)
	{
	case    DLL_PROCESS_ATTACH:
		g_hInstance = (HINSTANCE)hModule;				//インスタンスハンドルをグローバルにコピー
		break; 
	case    DLL_PROCESS_DETACH:
		break;
	default:
		break;
	}
	return TRUE;
}

/*-------------------------------------------------------------------------------------------------------*/
/* エクスポートする関数 */

int load( int status , char *in_path , void *in_reserved , MALLOC_FUNC in_mallocfunc , FREE_FUNC in_freefunc , COM_FUNC in_comfunc , STATUS_FUNC in_statusfunc )
{
	switch(status)
	{
	case STATUS_LOAD:
		{
			/* 各変数をグローバル変数へコピー */
			gicp_malloci = in_mallocfunc;
			gicp_freei = in_freefunc;
			gicp_com = in_comfunc;
			gicp_status = in_statusfunc;
			return TRUE;
		}
		break;
	case STATUS_UNLOAD:
		{
			return TRUE;
		}
		break;
	default:
		return FALSE;
		break;
	}
}

int configuration( HWND hWnd , unsigned short us_language )
{
	/* ここでは作成しませんが、利用する場合はDialogBox(又はDialogBoxParam)でダイアログボックスを作ってください。*/
	return FALSE;
}

int communication( char **lpp_buffer , int *lp_length )
{
	char *lp_sjis;
	int length;

	/* UTF-8文字列をShift_JISに変換する場合に必要な文字数("\0"含む) */
	length = charcode_utf2sjis( *lpp_buffer , NULL , 0 );
	/* メモリを確保 */
	lp_sjis = (char*)malloc( length*sizeof(char) );
	/* UTF-8文字列をShift_JISに変換して格納 */
	charcode_utf2sjis( *lpp_buffer , lp_sjis , length );
	/* 文字列の長さを取得 */

	/* GICPを処理する関数に送る */
	if(!gicp_communication( &lp_sjis , &length ))
	{
		/* GICP通信失敗 */
		free( lp_sjis );
		return FALSE;
	}

	/* GICPのメモリを解放 */
	gicp_free( *lpp_buffer );

	/* Shift_JIS文字列をUTF-8に変換する場合に必要な文字数("\0"含む) */
	length = charcode_sjis2utf( lp_sjis , NULL , 0 );
	/* GICPのメモリを確保 */
	*lpp_buffer = (char*)gicp_malloc( length*sizeof(char) );
	/* Shift_JIS文字列をUTF-8に変換して格納 */
	charcode_sjis2utf( lp_sjis , *lpp_buffer , length );

	/* メモリ解放 */
	free( lp_sjis );

	/* バッファサイズを代入 */
	*lp_length = length;
	return TRUE;
}

/*-------------------------------------------------------------------------------------------------------*/
/* GICPを処理する関数 */

/* GICP命令の大元の関数、命令ごとに割り振るって各関数に送る。 */
int gicp_communication( char **in_buffer , int *in_length )
{
	switch(gicp_switch( *in_buffer ))
	{
	case 1:
		return gicp_functiongetfunction( in_buffer , in_length );
		break;
	case 2:
		return gicp_functionexecution( in_buffer , in_length );
		break;
	default:
		return gicp_notsupport( in_buffer , in_length );
		break;
	}
}

/* 分岐する関数 */
/* これは非常に簡易な関数なので、本来はもう少しきちんとしたものを作る必要があると思います。*/
int gicp_switch( char *in_buffer )
{
	if(strstr( in_buffer ,  "<class>FUNCTION</class><order>GETFUNCTION</order>"))
	{
		/* FUNCTION GETFUNCTION 命令 */
		return 1;
	}
	else if(strstr( in_buffer ,  "<class>FUNCTION</class><order>EXECUTION</order>"))
	{
		/* FUNCTION EXECUTION 命令 */
		return 2;
	}
	else
	{
		return 0;
	}
}

/* GICP文字列を送信する */
int gicp_sendcom( char **in_buffer , int *in_length )
{
	int length;
	char *lp_utf8;

	/* Shift_JIS文字列をUTF-8に変換する場合に必要な文字数("\0"含む) */
	length = charcode_sjis2utf( *in_buffer , NULL , 0 );
	/* GICPのメモリ確保 */
	lp_utf8 = (char*)gicp_malloc( length*sizeof(char) );
	/* Shift_JIS文字列をUTF-8に変換して格納 */
	charcode_sjis2utf( *in_buffer , lp_utf8 , length );

	if(!gicp_com( &lp_utf8 , &length ))
	{
		/* GICP通信失敗 */
		gicp_free( lp_utf8 );	/* GICPのメモリを解放 */
		return FALSE;
	}

	/* メモリを解放 */
	free( *in_buffer );
	/* UTF-8文字列をShift_JISに変換する場合に必要な文字数("\0"含む) */
	length = charcode_utf2sjis( lp_utf8 , NULL , 0 );
	/* メモリ確保 */
	*in_buffer = (char*)malloc( length*sizeof(char) );
	/* UTF-8文字列をShift_JISに変換して格納 */
	charcode_utf2sjis( lp_utf8 , *in_buffer , length );

	/* GICPのメモリを解放 */
	gicp_free( lp_utf8 );

	/* バッファサイズを代入 */
	*in_length = length;
	return TRUE;
}

/* 各データを元にGICP文字列を作る関数 */
char *gicp_setstring( char *in_master , char *in_follower , char *in_witchcraft , char *in_incantation )
{
	char *buffer;
	int length;

	/* バッファのサイズを計算する */
	length = strlen( "<\?xml version=\"1.0\" encoding=\"UTF-8\"\?>\r\n<gicp version=\"1.0\">\r\n<master></master>\r\n<follower></follower>\r\n<witchcraft></witchcraft>\r\n<incantation></incantation>\r\n</gicp>" );
	length = length + strlen(in_master);
	length = length + strlen(in_follower);
	length = length + strlen(in_witchcraft);
	length = length + strlen(in_incantation);	

	/* バッファを確保する */
	buffer = (char*)malloc( (length+1)*sizeof(char) );

	/* バッファにも文字列をコピーする */
	sprintf( buffer , "<\?xml version=\"1.0\" encoding=\"UTF-8\"\?>\r\n<gicp version=\"1.0\">\r\n<master>%s</master>\r\n<follower>%s</follower>\r\n<witchcraft>%s</witchcraft>\r\n<incantation>%s</incantation>\r\n</gicp>" , in_master , in_follower , in_witchcraft , in_incantation );
	return buffer;
}

/* FUNCTION-GETFUNCTION命令を処理する関数 */
int gicp_functiongetfunction( char **in_buffer , int *in_length )
{
	char master[256];
	char follower[256];
	char witchcraft[256];
	char incantation[512];
	char *buffer;
	char *data;
	int length;

	char *p, *begin_p, *end_p;

	/* 各要素を作る */
	sprintf( master ,  "<place>SOMEONE</place><detail></detail>" );
	sprintf( follower ,  "<place>SOMEONE</place><detail></detail>" );
	sprintf( witchcraft , "<class>RETURN</class><order>RETURN</order>" );

	/* キーワードを抜き出します。 */
	/* かなりいい加減なXMLの処理なので、実際はもっと厳密に行ってください。 */
	p = strstr( *in_buffer , "<incantation>" );
	if(p)
	{
		begin_p = strstr( p , "<type>");
		end_p = strstr( p , "</type>" );

		/* 内容まではポインタを<data>分進めなければならない */
		begin_p = begin_p+strlen("<type>");

		/* 文字数にあわせてメモリを確保 */
		data = (char*)malloc( end_p-begin_p+1 );
		memset( data , 0 , end_p-begin_p+1 );
		memcpy( data , begin_p ,  end_p-begin_p );


		/* 『命令』の一覧は対応 */
		if(!strcmp(data,"ORDER"))
		{
			/* 『命令』の一覧を作成 */
			sprintf( incantation ,  "<result>OK</result><function><items><item no=\"1\">SAMPLE</item></items></function>" );

		}
		else if(!strcmp(data,"PARAM"))
		{
			/* 『内容』の一覧を作成 */
			sprintf( incantation ,  "<result>OK</result><function><order>SAMPLE</order><items><item no=\"1\">WINDOW</item><item no=\"2\">MESSAGE</item></items></function>" );

		}
		else
		{
			/* それ以外は対応していません */
			sprintf( incantation ,  "<result>NOTSUPPORTED</result>" );
		}

		/* 忘れずにメモリの解放 */
		free( data );
	}
	else
	{
		/* 必要な内容がたりていない */
		sprintf( incantation ,  "<result>BADREQUEST</result>" );
	}

	

	/* 各データを元にGICP文字列を作る */
	buffer = gicp_setstring( master , follower , witchcraft , incantation );
	length = strlen(buffer)+1;

	/* 元のメモリ解放してから代入 */
	free( *in_buffer );
	*in_buffer = buffer;
	*in_length = length;
	return TRUE;
}

/* FUNCTION-EXECUTION命令を処理する関数 */
int gicp_functionexecution( char **in_buffer , int *in_length )
{
	char master[256];
	char follower[256];
	char witchcraft[256];
	char incantation[256];
	char *buffer;
	int length;

	char *p;

	/* キーワードをチェックします。 */
	/* この時点でバグでなければヒットしていることはすでに分かっています。 */
	/* かなりいい加減なXMLの処理なので、実際はもっと厳密に行ってください。 */
	p = strstr( *in_buffer , "<incantation>" );

	/* ここで分岐 */
	if(strstr( p , "WINDOW"))
	{
		/* ウィンドウを表示 */
		window_createwindow( );
	}
	else if(strstr( p , "MESSAGE" ))
	{
		/* メッセージを送る */
		gicp_noticerequest();
	}

	/* GICP RETURN の作成 */
	sprintf( master , "<place>SOMEONE</place><detail></detail>" );
	sprintf( follower , "<place>SOMEONE</place><detail></detail>" );
	sprintf( witchcraft , "<class>RETURN</class><order>RETURN</order>" );
	sprintf( incantation ,  "<result>OK</result>" );

	/* 各データを元にGICP文字列を作る */
	buffer = gicp_setstring( master , follower , witchcraft , incantation );
	length = strlen(buffer)+1;

	/* 元のメモリ解放してから代入 */
	free( *in_buffer );
	*in_buffer = buffer;
	*in_length = length;
	return TRUE;
}

/* 未対応の命令が来た場合の対処 */
int gicp_notsupport( char **in_buffer , int *in_length )
{
	char master[256];
	char follower[256];
	char witchcraft[256];
	char incantation[512];
	char *buffer;
	int length;

	/* 各要素を作る */
	sprintf( master ,  "<place>SOMEONE</place><detail></detail>" );
	sprintf( follower ,  "<place>SOMEONE</place><detail></detail>" );
	sprintf( witchcraft , "<class>RETURN</class><order>RETURN</order>" );
	sprintf( incantation ,  "<result>NOTSUPPORTED</result>" );

	/* 各データを元にGICP文字列を作る */
	buffer = gicp_setstring( master , follower , witchcraft , incantation );
	length = strlen(buffer)+1;

	/* 元のメモリ解放してから代入 */
	free( *in_buffer );
	*in_buffer = buffer;
	*in_length = length;
	return TRUE;
}

/* NOTICE-REQUEST命令を送る関数 */

/* 注意:同梱の『思考エンジン』のサンプルには、この命令の処理は書かれていません。*/
/*		動作確認にはデフォルトのキャラクターを利用してください。*/
/*		もちろん、サンプルプログラムに独自に実装して試してみることはできます。 */
int gicp_noticerequest()
{
	char master[256];
	char follower[256];
	char witchcraft[256];
	char incantation[1024];
	char *buffer;
	int length;

	/* 各要素を作る */
	sprintf( master , "<place>OPTION</place><detail><name>SAMPLE</name></detail>" );
	sprintf( follower , "<place>MASCOT</place><detail></detail>" );
	sprintf( witchcraft , "<class>NOTICE</class><order>REQUEST</order>" );
	sprintf( incantation , "<event><information>MESSAGE</information><parameter></parameter></event><data><sender>サンプル</sender><string>サンプルオプションプログラムからメッセージを表示することもできます！</string><priority>DAUGHTER</priority></data><userdata></userdata>" );

	/* 各データを元にGICP文字列を作る */
	buffer = gicp_setstring( master , follower , witchcraft , incantation );
	length = strlen(buffer)+1;

	/* GICPを送信！*/
	gicp_sendcom( &buffer , &length );

	/* メモリ解放 */
	free( buffer );
	return TRUE;
}

/*-------------------------------------------------------------------------------------------------------*/
/* 文字コードを処理する関数 */

/* UTF-8をShift_JISに直接変換する関数はないが、Unicode(UTF-16)を介することで比較的簡単に実現できる */
/* buffer_sizeが0の時は必要なメモリ量を戻り値として返し、0以外の場合はlp_utfに変換して格納する */

/* Shift_JISからUTF-8に変換する */
int charcode_sjis2utf( char *lp_sjis , char *lp_utf8 , int buffer_size )
{
	wchar_t *lp_unicode;
	int length;

	/* UTF-8文字列をUnicodeに変換する場合に必要な文字数("\0"含む) */
	length = MultiByteToWideChar( CP_ACP , 0 , lp_sjis , -1 , NULL , 0 );
	/* メモリを確保 */
	lp_unicode = (wchar_t*)malloc( length*sizeof(wchar_t) );
	/* UTF-8文字列をUnicodeに変換して格納 */
	MultiByteToWideChar( CP_ACP , 0 , lp_sjis , -1 , lp_unicode , length );

	if(buffer_size==0)
	{
		/* Unicode文字列をUTF-8に変換する場合に必要な文字数("\0"含む) */
		length = WideCharToMultiByte( CP_UTF8 , 0 , lp_unicode , -1 , NULL , 0 , NULL , NULL );
		free( lp_unicode );
		return length;
	}
	else
	{
		/* Unicode文字列をUTF-8に変換して格納 */
		length = WideCharToMultiByte( CP_UTF8 , 0 , lp_unicode , -1 , lp_utf8 , buffer_size , NULL , NULL );
		free( lp_unicode );
		return length;
	}
}

/* UTF-8からShift_JISに変換する */
int charcode_utf2sjis( char *lp_utf8 , char *lp_sjis , int buffer_size )
{
	wchar_t *lp_unicode;
	int length;

	/* UTF-8文字列をUnicodeに変換する場合に必要な文字数("\0"含む) */
	length = MultiByteToWideChar( CP_UTF8 , 0 , lp_utf8 , -1 , NULL , 0 );
	/* メモリを確保 */
	lp_unicode = (wchar_t*)malloc( length*sizeof(wchar_t) );
	/* UTF-8文字列をUnicodeに変換して格納 */
	MultiByteToWideChar( CP_UTF8 , 0 , lp_utf8 , -1 , lp_unicode , length );

	if(buffer_size==0)
	{
		/* Unicode文字列をUTF-8に変換する場合に必要な文字数("\0"含む) */
		length = WideCharToMultiByte( CP_ACP , 0 , lp_unicode , -1 , NULL , 0 , NULL , NULL );
		free( lp_unicode );
		return length;
	}
	else
	{
		/* Unicode文字列をUTF-8に変換して格納 */
		length = WideCharToMultiByte( CP_ACP , 0 , lp_unicode , -1 , lp_sjis , buffer_size , NULL , NULL );
		free( lp_unicode );
		return length;
	}
}

/*-------------------------------------------------------------------------------------------------------*/
/* ウィンドウプログラム関連の関数 */

/* ウィンドウを作るとメッセージループで処理が止まってしまうのでスレッドで分けます。*/
int window_createwindow( )
{
	/*スレッドを作ります。*/
	_beginthread( window_thread , 0 , NULL );

	/*ウィンドウが表示している間に、*/
	/*マスコット本体が終了される場合を想定して、*/
	/*unlond関数からスレッドを終了させる機構を、*/
	/*持っているとより確実です。*/

	return TRUE;
}

/*ウィンドウのメッセージループをおくスレッドです。*/
void window_thread( void *lpvoid )
{
	MSG msg;
	WNDCLASSEXA wcex;
	HWND hWnd;

	/* ウィンドウクラスを登録 */
	memset( &wcex , 0 , sizeof(WNDCLASSEXA) );
	wcex.cbSize			= sizeof(WNDCLASSEXA);
	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= window_procedure;
	wcex.hInstance		= g_hInstance;
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszClassName	= "WindowClass";
	RegisterClassExA(&wcex);

	/* ウィンドウを作成 */
	hWnd = CreateWindowA( "WindowClass", "サンプルオプションプログラム" , WS_OVERLAPPEDWINDOW,CW_USEDEFAULT, 0, 400, 300, NULL, NULL, g_hInstance, NULL);
	if(!hWnd)
	{
		return;
	}

	ShowWindow(hWnd, SW_SHOW);
	UpdateWindow(hWnd);

	while(GetMessage(&msg, NULL, 0, 0)) 
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	return;
}

/* ウィンドウプロシージャ */
LRESULT CALLBACK window_procedure(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_PAINT:
		{
			PAINTSTRUCT ps;
			HDC hDC;
			char string[256];
			hDC = BeginPaint(hWnd, &ps);

			/* 表示処理 */
			GetWindowTextA( hWnd , string , 256 );
			TextOutA( hDC , 0 , 0 , string , strlen(string) );

			EndPaint(hWnd, &ps);
			return 0;
		}
		break;
	case WM_DESTROY:
		{
			/*ウィンドウを終了する場合は、*/
			/*メッセージループを終了してスレッドを終わらせます。*/
			PostQuitMessage(0);
			return 0;
		}
		break;
	default:
		return DefWindowProcA(hWnd, message, wParam, lParam);
		break;
	}
}