/*-----------------------------------------------------------------------------------------------------*/
/* ウィンドウメッセージで連携するプログラムのサンプル	2010/01/30 作成 2019/06/23 改訂*/

#include <stdio.h>
#include <windows.h>

/*-----------------------------------------------------------------------------------------------------*/
/* メモ書き */

/*
オプションプログラムの簡単なサンプルコードです。

Visual Stadio 2019 での利用を想定して記述しましたが、基本的には他の環境でも利用できると思います。
Win32 APIを使ったコードですが、MFCや.Net系でも扱い方は似たり寄ったりだと思います。

このソースコードはCで書かれていますがC++でコンパイルしても問題ないように記述されています。
ソースコードのうち以下のような記述部分はCでコンパイルしてもC++でコンパイルしても通るようにするためのものです。
#ifdef __cplusplus
extern "C" {
#endif

コード

#ifdef __cplusplus
}
#endif

ソースコードは流れを見やすくするためにエラー処理をほとんど行っていません。
実際にはきちんと要所でエラー処理を行ってください。

このプログラムはスレッドを使っているので、ランタイムライブラリはマルチスレッド系を選んで下さい。
*/

/*-----------------------------------------------------------------------------------------------------*/
/* 宣言 */

/* メニューのID登録 */
#define ID_SEND 1
#define ID_EXIT 2

/* バッファサイズ(アバウトな数字) */
#define MAX_STRING 512

/* プロトタイプ宣言 */
LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);

/* メイン関数 */
int APIENTRY WinMain( HINSTANCE hInstance , HINSTANCE hPrevInstance , LPTSTR lpCmdLine , int nCmdShow )
{
	MSG msg;
	WNDCLASSEX wcex;
	HWND hWnd;

	/* ウィンドウクラスの登録 */
	memset(&wcex,0,sizeof(WNDCLASSEX));
	wcex.cbSize			= sizeof(WNDCLASSEX);
	wcex.style			= CS_HREDRAW|CS_VREDRAW;
	wcex.lpfnWndProc	= WndProc;
	wcex.hInstance		= hInstance;
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszClassName	= "WindowClass";
	RegisterClassEx(&wcex);

	/* ウィンドウの作成 */
	hWnd = CreateWindow( "WindowClass" , "Sample" , WS_OVERLAPPEDWINDOW , CW_USEDEFAULT , 0 , 240 , 100 , NULL , NULL , hInstance , NULL );
	ShowWindow(hWnd, SW_SHOW );
	UpdateWindow(hWnd);

	/* ウィンドウメッセージループ */
	while(GetMessage(&msg, NULL, 0, 0)>0)
	{
		DispatchMessage(&msg);
	}
	return (int)msg.wParam;
}

/* ウィンドウのプロシージャー */
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_CREATE:
		{
			HMENU hMenu;
			HMENU hSubMenu;
			
			/* メニューの作成 */
			hMenu = CreateMenu();
			hSubMenu = CreateMenu();

			AppendMenu( hMenu , MF_POPUP, (UINT_PTR)hSubMenu , "メニュー" );
			AppendMenu( hSubMenu , MF_STRING , ID_SEND , "GICP命令を送る" );
			AppendMenu( hSubMenu , MF_STRING , ID_EXIT , "終了" );
			SetMenu( hWnd , hMenu );
			return TRUE;
		}
		break;
	case WM_COMMAND:
		{
			/* コマンド(メニュー)処理 */
			unsigned short wmId, wmEvent;
			wmId    = LOWORD(wParam);
			wmEvent = HIWORD(wParam);

			switch (wmId)
			{
			case ID_SEND:
				{
					/* GICP命令を送るが選択された場合の処理 */

					HWND f_hWnd;
					char *p;
					COPYDATASTRUCT cds;
					int result;
					char buffer[10];

					f_hWnd = FindWindow( "FyMascotSystem" , NULL );

					/* GICPLITE文字列  */
					/* 文字列はUTF-8(or ASCII)である必要があるので注意してください！ (Shift_JISでの日本語はダメ) */
					p = "<\?xml version=\"1.0\" encoding=\"UTF-8\"\?>\r\n<gicplite version=\"1.0\">\r\n<master><place>PROGRAM</place><detail></detail></master><follower><place>MASCOT</place><detail></detail></follower><witchcraft><class>NOTICE</class><order>REQUEST</order></witchcraft>\r\n<incantation>\r\n<event><information>MESSAGE</information><parameter></parameter></event>\r\n<data><sender>Sample!</sender><string>The sample program send a message.</string><priority>DAUGHTER</priority></data>\r\n<userdata></userdata></incantation>\r\n</gicplite>";

					/* 送信データを代入 */
					cds.lpData = p;				/* 送信文字列 */
					cds.cbData = (int)strlen(p)+1;		/* 送信文字列の長さ(\0も含む) */
					cds.dwData = (ULONG_PTR)hWnd;			/* このプログラムの受信側ウィンドウハンドル */

					/* Fy Mascot に送信 */
					result = (int)SendMessage( f_hWnd , WM_COPYDATA , 0L , (LPARAM)&cds );

					/* 結果 */
					sprintf( buffer , "[%d]" , result );
					MessageBox( hWnd , buffer , "結果コード" , MB_OK );
					return TRUE;
				}
				break;
			case ID_EXIT:
				{
					/* 終了が選択された場合の処理 */
					DestroyWindow( hWnd );
					return TRUE;
				}
				break;
			default:
				return DefWindowProc(hWnd, message, wParam, lParam);
				break;
			}
			return TRUE;
		}
		break;
	case WM_COPYDATA:
		{
			/* ここで『Fy Mascot』からの通信を受信する */
			((COPYDATASTRUCT*)lParam)->cbData;
			((COPYDATASTRUCT*)lParam)->lpData;
			return 200;
		}
		break;
	case WM_PAINT:
		{
			/* 描画 */
			/* このプログラムの本質とは関係ありません。 */
			PAINTSTRUCT ps;
			HDC hDC;
			char *p;

			hDC = BeginPaint(hWnd, &ps);
			p = "Sample!";
			TextOut( hDC , 10 , 10 , p , (int)strlen(p) );
			EndPaint(hWnd, &ps);
			return TRUE;
		}
		break;
	case WM_DESTROY:
		{
			PostQuitMessage(0);
			return 0;
		}
		break;
	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
		break;
	}
}